@@ -83,6 +83,13 @@ class MessageCore:
                self.sslContext.use_privatekey_file(sslKeyPath)
                self.sslContext.use_certificate_file(sslCertPath)
                self.sslContext.load_verify_locations(sslCACertPath)
+               try:
+#https://github.com/pyca/pyopenssl/issues/190  -  python3 addition
+                  self.sslContext.set_mode(SSL._lib.SSL_MODE_ENABLE_PARTIAL_WRITE |
+                                           SSL._lib.SSL_MODE_ACCEPT_MOVING_WRITE_BUFFER |
+                                           SSL._lib.SSL_MODE_AUTO_RETRY)
+               except:
+                  pass
 
          self.bindSocket = self.getMessageSocket(reuseAddress)
 
@@ -121,6 +128,13 @@ class MessageCore:
                self.sslContext = SSL.Context(SSL.TLSv1_METHOD)
                self.sslContext.set_verify(SSL.VERIFY_PEER,self.__verifyCert) # Demand a certificate
                self.sslContext.load_verify_locations(sslCACertPath)
+               try:
+#https://github.com/pyca/pyopenssl/issues/190  -  python3 addition
+                  self.sslContext.set_mode(SSL._lib.SSL_MODE_ENABLE_PARTIAL_WRITE |
+                                           SSL._lib.SSL_MODE_ACCEPT_MOVING_WRITE_BUFFER |
+                                           SSL._lib.SSL_MODE_AUTO_RETRY)
+               except:
+                  pass
 
 
    def __verifyCert(self,
@@ -197,9 +211,9 @@ class MessageCore:
       remoteHost = None
       try:
          channel,details = self.bindSocket.accept()
-      except socket.error,err:
+      except socket.error as e:
          self.logger.log(logging.ERROR,getLogMessage(traceback.format_exc()))
-         if err[0] in [EAGAIN,EWOULDBLOCK]:
+         if e.args[0] in [EAGAIN,EWOULDBLOCK]:
             channel = None
          else:
             raise
@@ -244,16 +258,16 @@ class MessageCore:
             listenerChannel.close()
          listenerChannel = None
          raise
-      except socket.gaierror,err:
+      except socket.gaierror as e:
          if logErrorMessage:
-            self.logger.log(logging.ERROR,getLogMessage("openListenerChannel: %s" % (err.args[1])))
+            self.logger.log(logging.ERROR,getLogMessage("openListenerChannel: %s" % (e.args[1])))
          if listenerChannel:
             listenerChannel.close()
          listenerChannel = None
-      except Exception,err:
-         if err.args[0] == 111:
+      except Exception as e:
+         if e.args[0] == 111:
             if logErrorMessage:
-               self.logger.log(logging.ERROR,getLogMessage("openListenerChannel: %s" % (err.args[1])))
+               self.logger.log(logging.ERROR,getLogMessage("openListenerChannel: %s" % (e.args[1])))
          else:
             if recordTraceback:
                self.logger.log(logging.ERROR,getLogMessage(traceback.format_exc()))
@@ -295,14 +309,14 @@ class MessageCore:
       while True:
          try:
             messageChunk = channel.recv(bufferSize)
-         except socket.error,err:
+         except socket.error as e:
             # Happens on non-blocking TCP socket when there's nothing to read
-            if isinstance(err.args,tuple):
-               if   err.args[0] in [ECONNRESET]:
+            if isinstance(e.args,tuple):
+               if   e.args[0] in [ECONNRESET]:
                   # Connection reset by peer
                   if not message:
                      message = None
-               elif not err.args[0] in [EAGAIN,EWOULDBLOCK]:
+               elif not e.args[0] in [EAGAIN,EWOULDBLOCK]:
                   self.logger.log(logging.ERROR,getLogMessage("Unexpected error in receiveNonBlockingMessage()"))
                   self.logger.log(logging.ERROR,getLogMessage(traceback.format_exc()))
             else:
@@ -312,8 +326,8 @@ class MessageCore:
          except (SSL.WantReadError, SSL.WantWriteError, SSL.WantX509LookupError):
             # Happens on non-blocking TCP/SSL socket when there's nothing to read
             break
-         except SSL.SysCallError, (errnum,errstr):
-            if errnum == -1:
+         except SSL.SysCallError as e:
+            if e.args[0] == -1:
                # Unexpected EOF
                if not message:
                   message = None
@@ -326,7 +340,7 @@ class MessageCore:
             break
          else:
             if messageChunk:
-               message += messageChunk
+               message += messageChunk.decode('utf-8')
             else:
                if not message:
                   message = None
@@ -347,6 +361,7 @@ class MessageCore:
       try:
          while bytesRemaining:
             messageChunk = channel.recv(bufferSize)
+            messageChunk = messageChunk.decode('utf-8')
             message += messageChunk
             bytesRemaining -= len(messageChunk)
             if messageChunk == "":
@@ -395,9 +410,13 @@ class MessageCore:
             transmittedLength = -1
       except (SSL.WantReadError, SSL.WantWriteError, SSL.WantX509LookupError):
          transmittedLength = 0
-      except Exception,err:
-         if isinstance(err.args,tuple):
-            if not err[0] in [EPIPE]:
+      except SSL.Error:
+         self.logger.log(logging.ERROR,getLogMessage("Unexpected SSL error in sendNonBlockingMessage(%s)" % (message[0:20])))
+         self.logger.log(logging.ERROR,getLogMessage(traceback.format_exc()))
+         transmittedLength = 0
+      except Exception as e:
+         if isinstance(e.args,tuple):
+            if not e.args[0] in [EPIPE]:
                self.logger.log(logging.ERROR,getLogMessage("Unexpected error in sendNonBlockingMessage(%s)" % (message[0:20])))
                self.logger.log(logging.ERROR,getLogMessage(traceback.format_exc()))
             else:
@@ -434,20 +453,20 @@ class MessageCore:
                   self.logger.log(logging.ERROR,getLogMessage("socket connection broken in sendBlockingMessage()"))
                   transmittedLength = -1
                   break
-      except SSL.Error,err:
-         if isinstance(err.args,tuple):
-            if not err[0] in [EPIPE]:
+      except SSL.Error as e:
+         if isinstance(e.args,tuple):
+            if not e.args[0] in [EPIPE]:
                self.logger.log(logging.ERROR,getLogMessage("Unexpected error in sendBlockingMessage(%s)" % (message[0:20])))
-               self.logger.log(logging.ERROR,getLogMessage(traceback.format_exc()))
+               self.logger.log(logging.ERROR,getLogMessage("%s %s" % (e.args[0][0][1],e.args[0][0][2])))
             else:
                self.logger.log(logging.ERROR,getLogMessage("sendBlockingMessage: Broken pipe"))
          else:
             self.logger.log(logging.ERROR,getLogMessage("Unexpected error in sendBlockingMessage(%s)" % (message[0:20])))
             self.logger.log(logging.ERROR,getLogMessage(traceback.format_exc()))
          transmittedLength = -1
-      except Exception,err:
-         if isinstance(err.args,tuple):
-            if not err[0] in [EPIPE]:
+      except Exception as e:
+         if isinstance(e.args,tuple):
+            if not e.args[0] in [EPIPE]:
                self.logger.log(logging.ERROR,getLogMessage("Unexpected error in sendBlockingMessage(%s)" % (message[0:20])))
                self.logger.log(logging.ERROR,getLogMessage(traceback.format_exc()))
             else:
@@ -504,7 +523,7 @@ class MessageCore:
                         channel.setblocking(0)
             else:
                self.logger.log(logging.ERROR,getLogMessage("Connection acceptHandshake failed.  No response."))
-      except Exception, err:
+      except Exception:
          self.logger.log(logging.ERROR,getLogMessage("Connection acceptHandshake failed.  Protocol mismatch?"))
          self.logger.log(logging.ERROR,getLogMessage("acceptHandshake(%s): %s" % (message.strip(),str(reply).strip())))
          self.logger.log(logging.ERROR,getLogMessage(traceback.format_exc()))
@@ -545,7 +564,7 @@ class MessageCore:
                   else:
                      channel.setblocking(0)
                   self.defaultBufferSize = int(reply.split()[-1])
-      except Exception, err:
+      except Exception:
          self.logger.log(logging.ERROR,getLogMessage("Connection initiateHandshake failed.  Protocol mismatch?"))
          self.logger.log(logging.ERROR,getLogMessage("initiateHandshake(%s): %s" % (message.strip(),str(reply).strip())))
          self.logger.log(logging.ERROR,getLogMessage(traceback.format_exc()))
